-- Copyright (C) 2012 Nicholas Carlson
--
-- Permission is hereby granted, free of charge, to any person obtaining a
-- copy of this software and associated documentation files (the "Software"),
-- to deal in the Software without restriction, including without limitation
-- the rights to use, copy, modify, merge, publish, distribute, sublicense,
-- and/or sell copies of the Software, and to permit persons to whom the
-- Software is furnished to do so, subject to the following conditions:
--
-- The above copyright notice and this permission notice shall be included in
-- all copies or substantial portions of the Software.
--
-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
-- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
-- AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-- LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-- FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-- DEALINGS IN THE SOFTWARE.


local camera = require 'lib.hump.camera'
local Sprite = require 'lib.sprite'
local vec2 = require 'lib.hump.vector'


local clouds = {}

function clouds:addrandomcloud()
  local cloud = {}

  local randx = love.math.random(-gWindow.width/2, gWindow.width/2)
  local randy = gWindow.height/4 + (love.math.random() * gWindow.height/5)

  cloud.startscale = -1.5
  cloud.endscale = 2.5

  cloud.startopacity = 0.74
  cloud.endopacity = 0
 
  cloud.startypos = randy
  cloud.endypos = randy + (256 * 8)

  cloud.startxpos = randx
  cloud.endxpos = randx + ((randx)/1 * 16)

  cloud.timer = 9
  cloud.acc = love.math.random() * 9

  local alpha = 1.0 - ((cloud.timer - cloud.acc) / cloud.timer)

  cloud.scale = cloud.startscale
  cloud.opacity = cloud.startopacity
  cloud.ypos = cloud.startypos
  cloud.xpos = cloud.startxpos

  cloud.scale = cloud.endscale * alpha + cloud.startscale * (1 - alpha)
  cloud.scale = math.exp(cloud.scale)

  cloud.opacity = cloud.endopacity * alpha + cloud.startopacity * (1 - alpha)

  local f = 6
  cloud.ypos = cloud.endypos * math.exp(f*alpha-f) + cloud.startypos * (1 - math.exp(f*alpha-f))

  cloud.xpos = cloud.endxpos * math.exp(3*alpha-3) + cloud.startxpos * (1 - math.exp(3*alpha-3))
  cloud.xpos = self:centerx(cloud.xpos)

  table.insert(self.clouds, cloud)
end

function clouds:init()
  self.clouds = {}

  self.sprites = {}
  self.sprites.cloud = Sprite("assets/cloud-512x512.png", 256, 256);

  for i=1,200 do
    self:addrandomcloud()
  end
end

function clouds:centerx(x)
  return x + gWindow.width/2
end

function clouds:update(dt)
  for _,cloud in pairs(self.clouds) do
    cloud.acc = cloud.acc + dt
    if cloud.acc >= cloud.timer then
      cloud.acc = cloud.acc - cloud.timer
    end

    local alpha = 1.0 - ((cloud.timer - cloud.acc) / cloud.timer)

    cloud.scale = cloud.startscale
    cloud.opacity = cloud.startopacity
    cloud.ypos = cloud.startypos
    cloud.xpos = cloud.startxpos

    cloud.scale = cloud.endscale * alpha + cloud.startscale * (1 - alpha)
    cloud.scale = math.exp(cloud.scale)

    cloud.opacity = cloud.endopacity * math.exp(3*alpha-3) + cloud.startopacity * (1 - math.exp(3*alpha-3))

    local f = 6
    cloud.ypos = cloud.endypos * math.exp(f*alpha-f) + cloud.startypos * (1 - math.exp(f*alpha-f))

    cloud.xpos = cloud.endxpos * math.exp(3*alpha-3) + cloud.startxpos * (1 - math.exp(3*alpha-3))
    cloud.xpos = self:centerx(cloud.xpos)
  end

end

function clouds:draw()
  local previous = { love.graphics.getColor() }

  for _,cloud in pairs(self.clouds) do
    love.graphics.setColor(255, 255, 255, 255 * cloud.opacity)
    self.sprites.cloud:draw(cloud.xpos, cloud.ypos + 512, 0, cloud.scale, cloud.scale)
  end

  love.graphics.setColor(unpack(previous))
end

local titlemenu = {}
titlemenu.__index = titlemenu

local function new()
  local v = {}
  setmetatable(v, titlemenu)
  return v
end

function titlemenu:enter(previous, ...)
  self.fonts.previous = love.graphics.getFont()

  love.graphics.setBackgroundColor(6, 27, 42, 255)
  love.graphics.setFont(love.graphics.newFont("assets/fonts/Cloudlike.ttf", 196))

  self.sprites.sky = Sprite("assets/nightsky-512x512.png", 0, 0);
  self.sprites.sky.image:setWrap("repeat", "repeat")

  clouds:init()

  self.bgm:init()
end

function titlemenu:leave()
  love.graphics.setFont(self.fonts.previous)
end

function titlemenu:init()
  self.sprites = {}
  self.fonts = {}

  self.bgm = {
    buffers = {
      love.audio.newSource("assets/nopantsdance.ogg", "static");
      love.audio.newSource("assets/nopantsdance.ogg", "static");
    };
    timer = {
      acc = 0;
      timestep = 11.411;
    };
    index = 1;
    update = function(self, dt)
      self.timer.acc = self.timer.acc + dt
      if self.timer.acc >= self.timer.timestep then
        -- self.timer.acc = self.timer.acc - self.timer.timestep
        self.timer.acc = 0
        local prev = self.index
        self.index = (self.index + 2) % 2 + 1;
        love.audio.play(self.buffers[self.index])
        love.audio.rewind(self.buffers[prev])
      end
    end;
    init = function (self)
      love.audio.setVolume(0.4)
      love.audio.play(self.buffers[self.index])
    end;
  };

  self.camera = camera(vec2(gWindow.width/2, gWindow.height/2), gWindow.width / 1280)
end

function titlemenu:update(dt)
  clouds:update(dt)
  self.bgm:update(dt)
end

function titlemenu:draw()
  self.sprites.sky:draw(0, 0, 0, gWindow.width/512, 1)

  self.camera:attach()

  love.graphics.printf("Steamy", 0, 128, gWindow.width, "center")
  clouds:draw()

  self.camera:detach()
end

function titlemenu:mousepressed(x, y, button)
end

function titlemenu:mousereleased(x, y, button)
end

function titlemenu:keypressed(key, code)
end

function titlemenu:keyreleased(key, code)
  if key == "escape" then
    love.event.quit()
  end
end

function titlemenu:quit()
end

return setmetatable(
  {
    new = new,
  },
  {__call = function(_,...) return new(...) end})